@define-color def_bg_color rgba(62, 68, 73, 1);
@define-color overlay_color rgba(52, 58, 63, 1);
@define-color def_green_color rgb(86, 179, 102);

.login_links link{
    color: @def_green_color;
}

.login_links link:hover{
    color: rgb(255,255,255);
}

.login_version_label{
    color: rgb(127,127,127);
}

label, entry{
    text-shadow: none;
}

label selection{
    background-color: @def_green_color;
}

.default_background{
    background-color: @def_bg_color;
    color: rgb(255,255,255);;
}

.overlay_background{
    background-color: @overlay_color;
    color: rgb(255,255,255);;
}

.menu_bar{
    border: none;
    box-shadow: none;
    background-color: @overlay_color;
    color: rgb(255,255,255);
}

.menu_bar menuitem window menu{
    border-radius: 0;
    box-shadow: none;
    border:none;
    color: rgb(255,255,255);
}

.top_menu_hamburger{
    margin-top: 2px;
    margin-bottom: 2px;
}

.text_hamburger_identation{
    margin-left: -20px;
}

.white_text{
    color: rgb(255,255,255);;
}

.default_style_switch{
    background-color: @overlay_color;
    border-color: @overlay_color;
}

.default_style_switch slider{
	border: transparent;
}

.default_style_switch:checked{
    background-color: @def_green_color;
    border-color: @def_green_color;
}

.default_style_btn{
    -gtk-icon-effect: none;
    -gtk-icon-source: none;
    outline: none;
    box-shadow: none;
    background-image: none;
    text-shadow: none;
    background-color: @def_bg_color;
    color: rgb(255,255,255);;
    padding: 10px 50px;
    border: 2px solid rgb(255,255,255);;
    border-radius: 20px;
    transition: all .115s ease-in-out;
}

.main_connect_btn{
    -gtk-icon-effect: none;
    -gtk-icon-source: none;
    outline: none;
    box-shadow: none;
    background-image: none;
    text-shadow: none;
    color: rgb(255,255,255);;
    background-color: transparent;
    border: 2px solid rgb(255,255,255);;
    padding: 10px 50px;
    border-radius: 20px;
    transition: all .115s ease-in-out;
}

.delete_active_profile_btn{
    -gtk-icon-effect: none;
    -gtk-icon-source: none;
    outline: none;
    box-shadow: none;
    background-image: none;
    text-shadow: none;
    color: rgb(255,255,255);;
    background-color: transparent;
    border: none;
    border-radius: 5px;
    transition: all .115s ease-in-out;
}   

.default_style_btn:hover, .main_connect_btn:hover, .delete_active_profile_btn:hover{
    -gtk-icon-effect: none;
    -gtk-icon-source: none;
    outline: none;
    box-shadow: none;
    background-image: none;
    text-shadow: none;

    border-color: @def_green_color;
    background-color: @def_green_color;
}

/* Dashboard notebook style*/
notebook.dashboard_notebook_style{
	outline:none;
}

notebook.dashboard_notebook_style header{
	-gtk-icon-source: none;
    background-image: none;
	outline:none;
	padding: 0;
    background-color: @overlay_color;
    border: none;
}

notebook.dashboard_notebook_style tabs{
    -gtk-icon-source: none;
    background-image: none;
	outline:none;
	padding: 0;
    margin: 0;
    border-bottom: 1px solid rgba(255,255,255, 0.3);
    border: none;
}

notebook.dashboard_notebook_style tabs tab{
	-gtk-icon-source: none;
    background-image: none;
    outline:none;
    border-color: transparent;
    box-shadow: none;
	padding: 0;
    margin: 0;
    transition: all .115s ease-in-out;
    border: none;
}

.default_main_tab_conf{
    outline:none;
    border: none;
    box-shadow: none;
    padding: 20px 70px;
    color: rgb(255,255,255);;
    transition: all .115s ease-in-out;
}

.dashboard_left_tab{
    border-top-right-radius: 80px;
    margin-right: -10px;

}
.dashboard_right_tab{
    border-top-left-radius: 80px;
    margin-left: -10px;
}
/* Dashboard notebook style*/

.active_tab{
    color: rgb(255,255,255);;
    background-color: @def_bg_color;
}

.inactive_tab{
    background-color: rgba(55, 61, 66, 1);
    color: gray;
}

.countries_content_style{
    padding: 5px 10px;
    transition: all 5s ease-in-out;
}

.profiles_tab_content{
    padding: 10px 15px;
}

.profile_grid{
    padding: 20px;
}

.profile_style_btn{
    -gtk-icon-effect: none;
    -gtk-icon-source: none;
    outline: none;
    box-shadow: none;
    background-image: none;
    text-shadow: none;

    padding: 5px 5px;
    background-color: transparent;
    border: none;
    border-radius: 0;
    transition: all .115s ease-in;
}
.profile_style_btn:hover{
    color: @def_green_color;
    padding-left: 15px;
    box-shadow: none;
}
.server_scroll_window{
    border: none;
}

.default_style_combobox{
    -gtk-icon-effect: none;
    -gtk-icon-source: none;
    background-image: none;
    text-shadow: none;
    outline: none;
    border-image-width: 0;
    border-top: none;
    border-left: none;
    border-right: none;
    border-bottom: 1px solid rgba(255,255,255, 0.3);
}
.default_style_combobox button{
    -gtk-icon-effect: none;
    -gtk-icon-source: none;
    background-image: none;
    text-shadow: none;
    box-shadow: none;
    outline: none;
    border-image-width: 0;
    background-color: @def_bg_color;
    color: rgb(255,255,255);;
    border-radius: 0;
    border: none;
}

.default_style_combobox button box arrow{
    color: rgb(255,255,255);;
}

.default_style_combobox button box cellview{
    color: rgb(255,255,255);
}

.default_style_combobox button:focus{
    -gtk-icon-effect: none;
    -gtk-icon-source: none;
}

.default_style_combobox window menu{
    border: none;
    box-shadow: none;
    border-radius: 0;
    background-color: @def_bg_color;
	color:rgb(255,255,255);;
}

.default_style_combobox window menu menuitem cellview{
	color:rgb(255,255,255);;
}

.server_list_bakground_row:hover, .menu_list_hover:hover, .default_style_combobox window menu menuitem:hover{
    background-color: @def_green_color;
}

.remove_borders{
    border: none;
}

.bottom_border{
    border-bottom: 1px solid rgba(255,255,255, 0.3);
}

/* Entry default style */
entry.default_style_entry{
    border-image:none;
    outline: none;
    box-shadow: none;
    background-color: @def_bg_color;
    color: rgb(255,255,255);;
    border-top: none;
    border-left: none;
    border-right: none;
    border-bottom: 1px solid rgba(255,255,255, 0.3);
}
entry.default_style_entry:focus{
    border-image:none;
    outline: none;
    box-shadow: none;
}
/* Entry default style */

/*
||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
----------------------- Settings Window -----------------------
|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
*/

.settings_label_grid{
    background-color: rgba(45, 51, 56, 1);
}
.settings_main_logo{
    /* padding: 30px 50px; */
    padding: 50px 30px 0px 50px;
}
.settings_main_label{
    /* padding: 20px 50px; */
    padding: 10px 50px 50px 50px;
    margin-left: 45px;
}

/* Settings notebook style*/
notebook.settings_notebook_style{
	outline:none;
}

notebook.settings_notebook_style header{
    -gtk-icon-source: none;
	outline:none;
	padding: 0;
    background-color: transparent;
    border: none;
}

notebook.settings_notebook_style tabs{
	outline:none;
	padding: 0;
	margin: 0;
}
notebook.settings_notebook_style tab{
    box-shadow: none;
    border: transparent;
    background-image: none;
	outline:none;
	padding: 0;
    margin: 0;
    transition: all .115s ease-in-out;
}

.default_notebook_grid_content{
    color: rgb(255,255,255);;
    background-color: @def_bg_color;
    padding: 30px 100px;
    padding-bottom: 200px;
}

.default_settings_tab_conf{
    padding: 15px 40px;
    transition: all .115s ease-in-out;
}

.split_tunnling_list{
    background-color: @overlay_color;
    color: rgb(255,255,255);;
    padding: 10px 10px;
    /* border-radius: 5px; */
}
.split_tunnling_list text{
    background-color: @overlay_color;
    color: rgb(255,255,255);;
    /* border-radius: 5px; */
}
